/****************************************************************/
/*	Source file for  SM28VLT32-HT flash API
	Platform:	IAR Workbench, MSWindows
	Target  :	SM470R1B1M-HT	
    
	Developed by		   : Varalakshmi B
	Developed on		   : 13th Dec 2012
	Release Ver		   : 1.0		*/
/****************************************************************/

#include "SM28VLT32.h"

/************** Header files for TMS470R1B1m *****************/
#include <intrinsics.h>
#include <TexasInstruments/iotms470r1b1m.h>
#include <TexasInstruments/tms470r1B1m_bit_definitions.h>


/*********************************************************************************/
/*	Name:	        Flash_Init  						 */
/*										 */
/*	Parameters:	None							 */
/*										 */
/*	Return Value:	None    						 */
/*										 */
/*	Descriptions:	Configures SPI2 port in TMS470R1B1m for Flash Programming*/
/*					Master Mode, Character Length -8,	 */
/*					Phase -1, Polarity -0			 */
/*										 */
/*********************************************************************************/
void Flash_Init(void)
{
    int data = 0; 
    // SPI 2 Setup for  TMS470R1B1m
    SPI2CTRL1= CHARLEN_8 | PRESCALE_1;      //PRESCALE_1, Char Len- 8
    SPI2CTRL2 |= MASTER;                    // Master Mode
    SPI2CTRL2 |= CLKMOD;                    // Internal clock  
    SPI2CTRL2 |= PHASE;			    // Phase -1, Polarity -0
    
    SPI2PC6 |= SOMI_FUN | SIMO_FUN | CLK_FUN; // SOMI SIMO SPICLK enable, Manual control for Chip Select
    SPI2PC1 |=   0x10;            // Configure Chip select pin as output
    SPI2PC4 |=   0x10;            // Make Chip Selecct High

    SPI2CTRL2 |= SPIEN;                    // Enable  SPI2  
    data = SPI2BUF;                        // Dummy Read to clear buffer     
}

/********************************************************************************/
/*	Name:	        Flash_Set_CS_Status  					*/
/*										*/
/*	Parameters:	None							*/
/*										*/
/*	Return Value:	None    						*/
/*										*/
/*	Descriptions:	Set/Reset Chip select pin of SPI2 port in TMS470R1B1m   */
/*										*/
/********************************************************************************/
void Flash_Set_CS_Status(int status)
{	// 
    if(status == 0)
    {
        // Make Chip Select low
        SPI2PC5 |=   0x10;
    }
    else
    {
        // Make Chip select high
        SPI2PC4 |=   0x10;	
    }
}

/********************************************************************************/
/*	Name:	        Flash_SPI_SendByte  					*/
/*										*/
/*	Parameters:	None							*/
/*										*/
/*	Return Value:	None    						*/
/*										*/
/*	Descriptions:	Send byte to flash over SPI2 port in TMS470R1B1m   	*/
/*										*/
/********************************************************************************/
unsigned char Flash_SPI_SendByte(unsigned char data)
{
    // Write byte to SPI2DAT1 register
    SPI2DAT1 = data;				
    while(!(SPI2CTRL3 & 0x01)){}        // Wait for RxFlag to get set    
    return (SPI2BUF & 0x000000FF);      // Read SPIBUF 
}

/********************************************************************************/
/*	Name:		Flash_ReadStatusRegister			        */
/*										*/
/*	Parameters:	Pointer to Status   				        */
/*										*/
/*	Return Value:	None  							*/
/*										*/
/*	Descriptions:	Reads Status Register by sending    			*/
/*					0x22 0x00 0x00  			*/
/*										*/
/********************************************************************************/
void Flash_ReadStatusRegister(int *Status)
{
    int data0 = 0;
    
    // Make Chip Select low
    Flash_Set_CS_Status(0);
    
    // Send Command ID 0x22
    data0 = Flash_SPI_SendByte(0x22);
    
    // Data byte on SO - status byte MSB
    data0 = Flash_SPI_SendByte(0);
    data0 = data0 <<8;
        
    // Data byte on SO - status byte LSB    
    data0 |= Flash_SPI_SendByte(0);   
    
    *Status = data0;
    
    // Make Chip select high
    Flash_Set_CS_Status(1);	 
}


/********************************************************************************/
/*	Name:		Flash_ClearError				        */
/*										*/
/*	Parameters:	None				   		        */
/*										*/
/*	Return Value:	None  							*/
/*										*/
/*	Descriptions:	Clears Command error and Invalid data error.		*/
/*		        This function needs to be called when Command error	*/
/*					or Invalid data error occurs. 		*/
/*										*/
/********************************************************************************/
void Flash_ClearError()
{	  
    int data0 = 0;

    // Make Chip select low
    Flash_Set_CS_Status(0);

    // Send 0x1F
    data0 = Flash_SPI_SendByte(0x1F);
    
    // Send 0x00
    data0 = Flash_SPI_SendByte(0);
    
    // Send 0x40
    data0 = Flash_SPI_SendByte(0x40);
    
    // Send dummy byte
    data0 = Flash_SPI_SendByte(0);
    
    // Make Chip select high
    Flash_Set_CS_Status(1);	 
}


/********************************************************************************/
/*	Name:		Flash_Write						*/
/*										*/
/*	Parameters:	Address, Data		   				*/
/*										*/
/*	Return Value:	Error code						*/
/*		        Zero on sucessful write					*/
/*			0x01 on Command error					*/
/*                      0x04 on Invalid Data error				*/
/*			0x40 on SPI Frame Error					*/	
/*										*/
/*	Descriptions:	Writes Data to given Address locaton.			*/
/*			Flash_ClearError should be called on error		*/
/*										*/
/********************************************************************************/
int Flash_Write(int Address, unsigned short Data)
{ 
    int error ;
    int data0 = 0;
    
    // Make chip select Low
    Flash_Set_CS_Status(0);
    
    // Send Command ID 0x17
    data0 = Flash_SPI_SendByte(0x17);
    
    // Send Address Byte 3
    data0 = Flash_SPI_SendByte((Address & 0xFF0000) >> 16);    
    
    // Send Address Byte 2
    data0 = Flash_SPI_SendByte((Address & 0xFF00) >> 8);
    
    // Send Address Byte 1
    data0 = Flash_SPI_SendByte(Address & 0xFF);
    
    // Send Data Byte MSB
    data0 = Flash_SPI_SendByte((Data & 0xFF00) >> 8);
    
    // Send Data Byte LSB
    data0 = Flash_SPI_SendByte(Data & 0xFF);
    
    // Send Dummy Byte
    data0 = Flash_SPI_SendByte(0);
    
    // Make Chip select high
    Flash_Set_CS_Status(1);      
   
    // Send 0xFF to get Quick status
    error = Flash_SendFF();    
    // If SPI frame error or Invalid data or Command Error , return   
    if((error & FLASH_QS_COMMAND_ERROR) || 
	   (error & FLASH_QS_INVALID_DATA_ERROR) ||
	   (error & FLASH_QS_SPI_FRAME_ERROR))
     return error; 
      
    // Poll till Write busy bit is reset
    while(error & FLASH_QS_WRITE_BUSY)
    {	
        error = Flash_SendFF();        
    }  
    // error status comes only after Write busy bit is reset
    error = Flash_SendFF();  

    return error;  
}


/********************************************************************************/
/*	Name:		Flash_EraseSector				        */
/*										*/
/*	Parameters:	Address. Any address in the sector to be erased	*/
/*										*/
/*	Return Value:	Error code						*/
/*			Zero on sucessful write					*/
/*			0x01 on Command error					*/
/*                      0x04 on Invalid Data error				*/
/*			0x40 on SPI Frame Error					*/	
/*										*/
/*	Descriptions:	Erases the sector in which given Address is present.	*/
/*			Flash_ClearError should be called on error		*/
/*										*/
/********************************************************************************/
int Flash_EraseSector(int Address)
{
    int error;   
    int data0 = 0;
    
    // Make chip select Low
    Flash_Set_CS_Status(0);
    
    // Send Command ID
    data0 = Flash_SPI_SendByte(0x19);

    // Send Address Byte 3
    data0 = Flash_SPI_SendByte((Address & 0xFF0000) >> 16);
    
    // Send Address Byte 2
    data0 = Flash_SPI_SendByte((Address & 0xFF00) >> 8);
    
    // Send Address Byte 1
    data0 = Flash_SPI_SendByte(Address & 0xFF);
    
    // Send Dummy Byte
    data0 = Flash_SPI_SendByte(0);
    
    // Make Chip select high
    Flash_Set_CS_Status(1);  
    
    // Send 0xFF to get Quick status
    error = Flash_SendFF();    
    // If SPI frame error or Invalid data or Command Error , return   
    if((error & FLASH_QS_COMMAND_ERROR) || 
     (error & FLASH_QS_INVALID_DATA_ERROR) ||
     (error & FLASH_QS_SPI_FRAME_ERROR))
     return error; 
      
    // Poll till Write busy bit is reset
    while(error & FLASH_QS_ERASE_BUSY)
    {	
        error = Flash_SendFF();        
    }  
    // error status comes only after Write busy bit is reset
    error = Flash_SendFF();  
    
    return error;    
}


/********************************************************************************/
/*	Name:		Flash_Read						*/
/*										*/
/*	Parameters:	Address, Pointer to data read				*/
/*										*/
/*	Return Value:	Error code						*/
/*			Zero on sucessful write					*/
/*			0x01 on Command error					*/
/*                      0x04 on Invalid Data error				*/
/*			0x40 on SPI Frame Error					*/	
/*										*/
/*	Descriptions:	Reads data in given address				*/
/*			Flash_ClearError should be called on error		*/
/*										*/
/********************************************************************************/
int Flash_Read(int Address, unsigned short *Data)
{
     int error = 0; 
     int data0 = 0;
     
     // Make Chip select low
     Flash_Set_CS_Status(0);
  
     // Send Comamnd ID 0x15
     data0 = Flash_SPI_SendByte(0x15);
    
     // Send Address Byte 3
     data0 = Flash_SPI_SendByte((Address & 0xFF0000) >> 16);
     
     // Send Address Byte 2
     data0 = Flash_SPI_SendByte((Address & 0xFF00) >> 8);
     
     // Send Address Byte 1
     data0 = Flash_SPI_SendByte(Address & 0xFF);
     
     // Send dummy byte
     data0 = Flash_SPI_SendByte(0);
     
     // Data byte on SO - MSB
     data0 = Flash_SPI_SendByte(0);
     data0 = data0 <<8;
     
     // Data byte on SO - LSB
     data0 |= Flash_SPI_SendByte(0);
    
     // Make Chip select high
     Flash_Set_CS_Status(1);	
  
     // Assign data byte
     *Data = data0;   
        
     // Send 0xFF to get Quick status
     error = Flash_SendFF();
     // If SPI frame error or Invalid data or Command Error , return
     if((error & FLASH_QS_COMMAND_ERROR) || 
	   (error & FLASH_QS_INVALID_DATA_ERROR) ||
	   (error & FLASH_QS_SPI_FRAME_ERROR))
       return error; 
        
     return 0;       
}


/********************************************************************************/
/*	Name:		Flash_SendFF						*/
/*										*/
/*	Parameters:	None							*/
/*										*/
/*	Return Value:	Quick status						*/
/*			Zero when idle, Status update on any operation		*/
/*			Bit0- Command error			    		*/
/*                      Bit1- Read error 	        			*/
/*			Bit2- Invalid Data error		 		*/
/*			Bit3- Device Busy					*/
/*			Bit4- Erase Busy	    				*/
/*			Bit5- Write Busy 					*/
/*			Bit6- SPI Frame Error					*/
/*			Bit7- Unused						*/
/*										*/
/*	Descriptions:	Reads Quick Status after any operation to Flash		*/
/*			This function gives error after device busy bit is reset*/
/*																				*/
/********************************************************************************/
int Flash_SendFF()
{  
     int error;

     // Make Chip select low
     Flash_Set_CS_Status(0);
	 
     // Send 0xFF to get Quick status
     error = Flash_SPI_SendByte(0xFF);
  
     // Make Chip select high
     Flash_Set_CS_Status(1);  
     
     return error;
}






